@@ -198,19 +198,24 @@ get_levels <- function(x) {
   out
 }
 
-has_lvls <- function(info)
+has_lvls <- function(info) {
   !vapply(info, function(x) all(is.na(x$values)), c(logic = TRUE))
+}
 
 strings2factors <- function(x, info) {
   check_lvls <- has_lvls(info)
-  if (!any(check_lvls))
+  if (!any(check_lvls)) {
     return(x)
+  }
   info <- info[check_lvls]
+  vars <- names(info)
+  info <- info[vars %in% names(x)]
   for (i in seq_along(info)) {
     lcol <- names(info)[i]
-    x[, lcol] <- factor(as.character(getElement(x, lcol)),
-                        levels = info[[i]]$values,
-                        ordered = info[[i]]$ordered)
+    x[, lcol] <-
+      factor(as.character(x[[lcol]]),
+             levels = info[[i]]$values,
+             ordered = info[[i]]$ordered)
   }
   x
 }
@@ -223,28 +228,22 @@ strings2factors <- function(x, info) {
 # data frame with one missing value, that element of the list column will
 # be counted as complete.
 n_complete_rows <- function(x) {
-  list_cols <- purrr::map_lgl(x, is.list)
-  list_cols <- names(list_cols)[list_cols]
+  is_list_col <- purrr::map_lgl(x, is.list)
+  pos_list_cols <- which(is_list_col)
 
-  if (length(list_cols) > 0) {
-    # replace with logical vector (with possible missings) for calculations
-    x[, list_cols] <- map_dfc(list_cols, convert_to_logical, x = x)
+  for (pos_list_col in pos_list_cols) {
+    x[[pos_list_col]] <- purrr::map_lgl(x[[pos_list_col]], flatten_na)
   }
+
   sum(complete.cases(x))
 }
 
-insert_na <- function(x) {
-  has_miss <- all(is.na(x))
-  if (has_miss) {
-    res <- NA
+flatten_na <- function(x) {
+  if (all(is.na(x))) {
+    NA
   } else {
-    res <- FALSE
+    FALSE
   }
-  res
-}
-
-convert_to_logical <- function(col, x) {
-  map_lgl(x[[col]], insert_na)
 }
 
 ## short summary of training set
@@ -444,7 +443,7 @@ is_trained <- function(x)
 #' @keywords internal
 #' @rdname recipes-internal
 sel2char <- function(x) {
-  map_chr(x, to_character)
+  unname(map_chr(x, to_character))
 }
 
 to_character <- function(x) {
@@ -595,3 +594,20 @@ tidyr_new_interface <- function() {
   utils::packageVersion("tidyr") > "0.8.99"
 }
 
+# ------------------------------------------------------------------------------
+# For all imputation functions that substitute elements into an existing vector:
+# vctrs's cast functions would be better but we'll deal with the known cases
+# to avoid a dependency.
+
+cast <- function(x, ref) {
+  if (is.factor(ref)) {
+    x <- factor(x, levels = levels(ref), ordered = is.ordered(ref))
+  } else {
+    if (is.integer(ref) & !is.factor(ref)) {
+      x <- as.integer(round(x, 0))
+    }
+  }
+  x
+}
+
+
